//
// TinyPTC by Gaffer
// www.gaffer.org/tinyptc
//

#include "../stdafx.h"
#include <timeapi.h>
#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include "../tinyptc.h"
#include "Utils.h"


//-----------------------------------------------------------------
// Our Functions
//-----------------------------------------------------------------
#include "../sinusscroll/logo_raw.h"
#include "../sinusscroll/logo_pal.h"
#include "../sinusscroll/foot_raw.h"
#include "../sinusscroll/foot_pal.h"

int ZOOM = 2;
int WIDTH = 320;
int HEIGHT = 240;
#define SIZE 640*480
// Setup a buffer for our reading and writing (FrameBuffer)
int noise;
int carry;
unsigned int x, y, position;
int seed = 0x12345;
union PIXEL pixel[SIZE];
int *buffer = (int*)pixel;
int plasma[SIZE];
rgb palette[256];
int paletteShift = 0;

#define PI 3.141593

#define SCROLLTXT
#include "../sinusscroll/FX/Scrolls.h"

// Function declares
void Cls(int rgb);
void WritePixel(int x, int y, int rgb);
//
void initSize();
void initfx();
void render_effect();
void initPalette();
void initPlasma();
void generatePlasma();

int main()
{
	DWORD Temps_Ecoule = 0;	//permet de synchroniser les FPS (images par seconde)
													// Open our window/screen
	int quit;
	timeBeginPeriod(1);

	ZOOM = 2;

	initSize();

	if (!ptc_open("test", WIDTH, HEIGHT)) return 1;

	InitSinus();
	initPalette();
	initPlasma();

	quit = 0;
	do
	{
		//on lance l'affichage en synchronisant les FPS
		if (timeGetTime() - Temps_Ecoule >= (1000 / 60))	//60 fps (1000 ms /16.6 ms)
		{
			Temps_Ecoule = timeGetTime();
			render_effect();
		}

	} while (!quit);

	timeEndPeriod(1);
}

//-----------------------------------------
void initSize(void)
{
	WIDTH *= ZOOM;
	HEIGHT *= ZOOM;
	ptfont = font1;
	ptfont2 = font2;
	SINAMPL *= ZOOM;
}

void render_effect()
{
	Cls(0); // clear screen
	generatePlasma();
	DoScrollText();

	ptc_update(pixel); // update screen with DDraw
}

void initPalette()
{
	//https://lodev.org/cgtutor/plasma.html
	for (int x = 0; x < 256; x++)
	{
		//use HSVtoRGB to vary the Hue of the color through the palette
		//palette[x] = hsv2rgb(MAKEHSV(x, 255, 255));

		palette[x].r = (unsigned char)(128.0 + 128 * sin(3.1415 * x / 32.0));
		palette[x].g = (unsigned char)(128.0 + 128 * sin(3.1415 * x / 64.0));
		palette[x].b = (unsigned char)(128.0 + 128 * sin(3.1415 * x / 128.0));
	}
}

void initPlasma()
{
	for (y = 0; y < HEIGHT; y++)
	{
		for (x = 0; x < WIDTH; x++)
		{
			noise = (int)((
				128.0 + (128.0 * sin(x / 16.0))
				+ 128.0 + (128.0 * sin(y / 32.0))
				+ 128.0 + (128.0 * sin(sqrt((double)((x - WIDTH / 2.0)* (x - WIDTH / 2.0) + (y - HEIGHT / 2.0) * (y - HEIGHT / 2.0))) / 8.0))
				+ 128.0 + (128.0 * sin(sqrt((double)(x * x + y * y)) / 8.0))) / 2.0);// / 4.0f
			position = (y*WIDTH) + x;
			plasma[position] = noise;
		}
	}
}

void generatePlasma()
{
	if (seed++ > 0x0)
	{
		paletteShift++;
		seed = 0;
	}
	for (y = 0; y < HEIGHT; y++)
	{
		for (x = 0; x < WIDTH; x++)
		{
			position = (y*WIDTH) + x;
			pixel[position].colors = palette[(plasma[position] + paletteShift) % 256];
		}
	}
}

//--------------------------------------------
// Common Functions
//--------------------------------------------

// clear screen
void Cls(int rgb)
{
	int x;
	int y;
	//Do cls stuff
	for (y = 0; y<HEIGHT; y++)
	{
		for (x = 0; x<WIDTH; x++)
		{
			pixel[(y*WIDTH + x)].color = rgb;
		}
	}
}

// plot a pixel
void WritePixel(int x, int y, int rgb)
{
	//Do write pixel
	if (x>-1)
	{
		if (x<WIDTH)
		{
			if (y>-1)
			{
				if (y<HEIGHT)
				{
					pixel[(y*WIDTH + x)].color = rgb;
				}
			}
		}
	}
}
