//
// TinyPTC by Gaffer
// www.gaffer.org/tinyptc
//

#include "../stdafx.h"
#include <timeapi.h>
#include <stdio.h>
#include <stdlib.h>
#include <time.h>
#include <math.h>
#include "../tinyptc.h"

#define PI 3.141593

typedef struct
{
	int r, g, b;
} col;

//--- effects
#define STARS2
#define WRITER
#define SCROLLTXT
#define IMAGEBLT

// Function declares
void Cls(int rgb);
void WritePixel(int x, int y, int rgb);
//
void initSize();
void initfx();
void render_effect();
#ifdef IMAGEBLT
void DrawImage(unsigned char* img, int xpos, int ypos, int sx, int sy);
void DrawImage2(unsigned char* img, int xpos, int ypos, int sx, int sy);
#endif

//-----------------------------------------------------------------
// Our Functions
//-----------------------------------------------------------------
#include "logo_raw.h"
#include "logo_pal.h"
#include "foot_raw.h"
#include "foot_pal.h"

int ZOOM = 2;
int WIDTH = 320;
int HEIGHT = 240;
#define SIZE 640*480
// Setup a buffer for our reading and writing (FrameBuffer)
static int buffer[SIZE];

//----------- draw bitmap 
#ifdef STARS2
#include "FX/Starfields.h"
#endif

#ifdef SCROLLTXT
#include "FX/Scrolls.h"
#endif
#ifdef WRITER
#include "FX/Writers2.h"
#endif

#ifdef IMAGEBLT
// var bitmap
int logox = 320;
int logoy = 140;
int footx = 320;
int footy = 13;
unsigned char* ptlogo;
unsigned char* ptfoot;
int speedfade = 0, speedfade2 = 0, endfade = 0, endfade2 = 0;
unsigned char logo_r[256], logo_g[256], logo_b[256];
unsigned char foot_r[256], foot_g[256], foot_b[256];
unsigned char clogo_r[256], clogo_g[256], clogo_b[256];
unsigned char cfoot_r[256], cfoot_g[256], cfoot_b[256];
#endif


int main()
{
	DWORD Temps_Ecoule = 0;	//permet de synchroniser les FPS (images par seconde)
													// Open our window/screen
	int quit;
	timeBeginPeriod(1);

	ZOOM = 2;

	initSize();
	// Open our window/screen
	if (!ptc_open("VrS! HORIZON #2 REMAKE", WIDTH, HEIGHT)) return 1;

	initfx();

	quit = 0;
	do
	{
		/*MSG msg;
		while (PeekMessage(&msg, NULL, 0, 0, PM_NOREMOVE))
		{
			if (GetMessage(&msg, NULL, 0, 0)>0)
			{
				TranslateMessage(&msg);
				DispatchMessage(&msg);
			}
			else
			{
				quit = 1;
				break;
			}
		}*/

		//on lance l'affichage en synchronisant les FPS
		if (timeGetTime() - Temps_Ecoule >= (1000 / 60))	//60 fps (1000 ms /16.6 ms)
		{
			Temps_Ecoule = timeGetTime();
			render_effect();
		}

	} while (!quit);

	timeEndPeriod(1);
	return 0;
}

//-----------------------------------------
void initSize(void)
{
	WIDTH *= ZOOM;
	HEIGHT *= ZOOM;
#ifdef WRITER
	WRITERSTARTX *= ZOOM;
	WRITERSTARTY *= ZOOM;
#endif
	ptfont = font1;
	ptfont2 = font2;
	SINAMPL *= ZOOM;
#ifdef IMAGEBLT
	ptlogo = logo_raw;
	ptfoot = foot_raw;
#endif
}

void initfx()
{
#ifdef STARS2
	InitStars();
#endif
#ifdef WRITER
	InitWriter();
#endif
#ifdef SCROLLTXT
	InitSinus();
#endif

#ifdef IMAGEBLT
	int i;
	for (i = 0; i<255; i++)
	{
		//Palette array name
		logo_r[i] = logo_pal[i * 3];
		logo_g[i] = logo_pal[i * 3 + 1];
		logo_b[i] = logo_pal[i * 3 + 2];
		clogo_r[i] = 255; // start white
		clogo_g[i] = 255;
		clogo_b[i] = 255;
		foot_r[i] = foot_pal[i * 3];
		foot_g[i] = foot_pal[i * 3 + 1];
		foot_b[i] = foot_pal[i * 3 + 2];
		cfoot_r[i] = 255;
		cfoot_g[i] = 255;
		cfoot_b[i] = 255;
	}
#endif
}

void render_effect()
{
	Cls(0); // clear screen
#ifdef STARS2
	DrawStars();
#endif
#ifdef WRITER
	if (affichelogo == 1) DoWriter();
#endif
#ifdef SCROLLTXT
	DoScrollText();
#endif
#ifdef IMAGEBLT
	DrawImage2(ptlogo, -1 * ZOOM, 23 * ZOOM, logox, logoy); // strange bug : shift to the right if x=0
	DrawImage(ptfoot, -1 * ZOOM, 26 * ZOOM + 11 * ZOOM + HEIGHT - 50 * ZOOM - footy*ZOOM, footx, footy);
#endif

	ptc_update(buffer); // update screen with DDraw
}
//--------------------------------------------
// Common Functions
//--------------------------------------------

// clear screen
void Cls(int rgb)
{
	int x;
	int y;
	//Do cls stuff
	for (y = 0; y<HEIGHT; y++)
	{
		for (x = 0; x<WIDTH; x++)
		{
			buffer[(y*WIDTH + x)] = rgb;
		}
	}
}

// plot a pixel
void WritePixel(int x, int y, int rgb)
{
	//Do write pixel
	if (x>-1)
	{
		if (x<WIDTH)
		{
			if (y>-1)
			{
				if (y<HEIGHT)
				{
					buffer[(y*WIDTH + x)] = rgb;
				}
			}
		}
	}
}

#ifdef IMAGEBLT
// to do: mix the 2 routines in one
void DrawImage(unsigned char* img, int xpos, int ypos, int sx, int sy)
{
	int x, y, pixel, col, txp = 0, typ = 0;

	//flashing
	if (affichelogo == 1 && endfade<255)
	{
		if (speedfade == 1)
		{
			for (x = 0; x<255; x++)
			{
				if (cfoot_r[x]>foot_r[x]) cfoot_r[x] -= 0x11;
				if (cfoot_r[x]<foot_r[x]) cfoot_r[x] += 0x11;
				if (cfoot_g[x]>foot_g[x]) cfoot_g[x] -= 0x11;
				if (cfoot_g[x]<foot_g[x]) cfoot_g[x] += 0x11;
				if (cfoot_b[x]>foot_b[x]) cfoot_b[x] -= 0x11;
				if (cfoot_b[x]<foot_b[x]) cfoot_b[x] += 0x11;
			}
			speedfade = 0;
			endfade++;
		}
		else {
			speedfade = 1;
		}
	}

	for (y = 0; y<sy; y++)
	{
		txp = 0;
		for (x = 0; x<sx; x++)
		{
			pixel = *(img + x + y*sx);
			col = RGB(cfoot_b[pixel], cfoot_g[pixel], cfoot_r[pixel]);
			if (affichelogo == 0) pixel = 0;
			if (pixel != 0)
			{
				WritePixel(txp + xpos, typ + ypos, col);
				if (ZOOM == 2)
				{
					WritePixel(txp + xpos + 1, typ + ypos, col);
					WritePixel(txp + xpos + 1, typ + ypos + 1, col);
					WritePixel(txp + xpos, typ + ypos + 1, col);
				}
			}
			txp += ZOOM;
		}
		typ += ZOOM;
	}
}

void DrawImage2(unsigned char* img, int xpos, int ypos, int sx, int sy)
{
	int x, y, pixel, col, txp = 0, typ = 0;

	//flashing
	if (affichelogo == 1 && endfade2<255)
	{
		if (speedfade2 == 1)
		{
			for (x = 0; x<255; x++)
			{
				if (clogo_r[x]>logo_r[x]) clogo_r[x] -= 0x11;
				if (clogo_r[x]<logo_r[x]) clogo_r[x] += 0x11;
				if (clogo_g[x]>logo_g[x]) clogo_g[x] -= 0x11;
				if (clogo_g[x]<logo_g[x]) clogo_g[x] += 0x11;
				if (clogo_b[x]>logo_b[x]) clogo_b[x] -= 0x11;
				if (clogo_b[x]<logo_b[x]) clogo_b[x] += 0x11;
			}
			speedfade2 = 0;
			endfade2++;
		}
		else {
			speedfade2 = 1;
		}
	}

	for (y = 0; y<sy; y++)
	{
		txp = 0;
		for (x = 0; x<sx; x++)
		{
			pixel = *(img + x + y*sx);
			col = RGB(clogo_b[pixel], clogo_g[pixel], clogo_r[pixel]);
			if (affichelogo == 0) pixel = 0;
			if (pixel != 0)
			{
				WritePixel(txp + xpos, typ + ypos, col);
				if (ZOOM == 2)
				{
					WritePixel(txp + xpos + 1, typ + ypos, col);
					WritePixel(txp + xpos + 1, typ + ypos + 1, col);
					WritePixel(txp + xpos, typ + ypos + 1, col);
				}
			}
			txp += ZOOM;
		}
		typ += ZOOM;
	}
}
#endif
